import * as React from 'react';
import { noop } from 'lodash';
import classNames from 'classnames';

import { shallowEqual } from './utils';
// import FocusManager from './focus_manager';
// @ts-ignore
import gs from '@ladifire-css/sass/index.scss';

export interface ListItemProps {
  ariaLabel?: string;
  ariaExpanded?: boolean;
  id: string;
  children?: any;
  onFocusEnter?: any;
  onFocusWithin?: any;
  onFocusLeave?: any;
  onHeightChange?: any;
  onMouseEnter?: any;
  onMouseOver?: any;
  onMouseLeave?: any;
  onMouseDown?: any;
  onMouseUp?: any;
  className?: string;
  style: React.CSSProperties;
  role?: string;
  height?: number;
  validity: boolean;
  hasFocus?: boolean;
  hasFocusWithin?: boolean;
  isMouseDownOnItem?: boolean;
  shouldHorizontallyScroll?: boolean;
}

export class ListItem extends React.Component<ListItemProps> {
  static displayName = 'ListItem';

  static defaultProps = {
    ariaLabel: undefined,
    children: undefined,
    onFocusEnter: noop,
    onFocusWithin: noop,
    onFocusLeave: noop,
    onHeightChange: noop,
    onMouseEnter: noop,
    onMouseOver: noop,
    onMouseLeave: noop,
    onMouseDown: noop,
    onMouseUp: noop,
    className: '',
    style: {},
    role: 'listitem',
    height: undefined,
    validity: true,
    hasFocus: false,
    hasFocusWithin: false,
    isMouseDownOnItem: false,
    shouldHorizontallyScroll: false,
  };
  private mutationObserver: MutationObserver | null;
  private node: any;
  private justFocused: boolean;
  private focusTimerId: any;

  constructor( props: ListItemProps ) {
    super( props );

    this.updateHeight = this.updateHeight.bind( this );
    this.setRef = this.setRef.bind( this );
    this.onFocusEnter = this.onFocusEnter.bind( this );
    this.onFocusWithin = this.onFocusWithin.bind( this );
    this.onFocusLeave = this.onFocusLeave.bind( this );
    this.onMouseEnter = this.onMouseEnter.bind( this );
    this.onMouseOver = this.onMouseOver.bind( this );
    this.onMouseLeave = this.onMouseLeave.bind( this );
    this.onMouseDown = this.onMouseDown.bind( this );
    this.onMouseUp = this.onMouseUp.bind( this );
    this.mutationObserver = new MutationObserver( this.updateHeight );
    this.node = null;
    this.justFocused = false;
  }

  componentDidMount() {
    if ( this.node && this.mutationObserver ) {
      this.mutationObserver.observe( this.node, {
        childList: true,
        subtree: true,
        attributes: true,
        characterData: true,
      } );
      this.node.addEventListener( 'load', this.updateHeight, true );
    }
    this.updateHeight();
    this.maybeFocusNode();
  }

  shouldComponentUpdate( nextProps: ListItemProps ) {
    if ( ! shallowEqual( nextProps.style, this.props.style ) ) return true;
    if ( ! shallowEqual( nextProps.children.props, this.props.children.props ) ) return true;
    if ( nextProps.children.key !== this.props.children.key ) return true;
    if ( ! nextProps.validity ) return true;
    // if ( nextProps.hasFocus !== this.props.hasFocus ) return true;
    if ( nextProps.isMouseDownOnItem !== this.props.isMouseDownOnItem ) return true;
    // if ( nextProps.hasFocusWithin !== this.props.hasFocusWithin ) return true;
    return false;
  }

  componentDidUpdate() {
    this.updateHeight();
    this.maybeFocusNode();
  }

  componentWillUnmount() {
    if ( this.mutationObserver ) {
      this.mutationObserver.disconnect();
      this.mutationObserver = null;
    }
  }

  onFocusEnter( r: any ) {
    const a = this;
    const n = r.relatedEvent;
    if ( n && 'mousedown' === n.type ) return;
    clearTimeout( this.focusTimerId );
    this.justFocused = true;
    this.focusTimerId = setTimeout( function() {
      a.justFocused = false;
    }, 250 );
    this.props.onFocusEnter(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  onFocusWithin( r: any ) {
    const a = this;
    const n = r.relatedEvent,
      s = r.target,
      l = r.currentTarget;
    if ( n && 'mousedown' === n.type ) return;
    if ( s === l ) {
      clearTimeout( this.focusTimerId );
      this.justFocused = true;
      this.focusTimerId = setTimeout( function() {
        a.justFocused = false;
      }, 250 );
    }
    this.props.onFocusWithin(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  onFocusLeave = () => {
    this.props.onFocusLeave();
  };

  onMouseEnter( r: any ) {
    if ( this.justFocused ) return;
    const a = Object.assign(
      {
        id: this.props.id,
      },
      r
    );
    this.props.onMouseEnter( a );
  }

  onMouseOver( r: any ) {
    if ( this.justFocused ) return;
    this.props.onMouseOver(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  onMouseLeave( r: any ) {
    this.props.onMouseLeave(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  onMouseDown( r: any ) {
    this.props.onMouseDown(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  onMouseUp( r: any ) {
    this.props.onMouseUp(
      Object.assign(
        {
          id: this.props.id,
        },
        r
      )
    );
  }

  setRef( r: any ) {
    this.node = r;
  }

  updateHeight() {
    if ( ! this.node ) return;
    const r = this.node.getBoundingClientRect(),
      a = r.height;
    const needUpdate = Math.abs( a - (this.props.height || 0) ) > 0.5 || ! this.props.validity;
    if ( needUpdate ) {
      this.props.onHeightChange( this.props.id, a );
    }
  }

  maybeFocusNode() {
    this.props.hasFocus && this.node && this.node.focus();
  }

  determineTabIndex() {
    if ( this.props.isMouseDownOnItem ) return;
    if ( this.props.hasFocusWithin ) return 0;
    return -1;
  }

  render() {
    if ( ! this.props.children ) return null;

    const rootClass = classNames(
      gs.position__absolute,
      gs.width__100_percent,
      gs.pseudo_focus__outline_none, {
        [gs.width__auto]: this.props.shouldHorizontallyScroll,
      },
      this.props.className
    );

    // const rootClass = classNames(
    //   'c_virtual_list__item',
    //   {
    //     'c_virtual_list__item__focus': this.props.hasFocus,
    //     'c_virtual_list__item__auto_width': this.props.shouldHorizontallyScroll,
    //   },
    //   this.props.className
    // );

    return (
      <div
        role={ this.props.role }
        ref={ this.setRef }
        style={ this.props.style }
        aria-label={ this.props.ariaLabel }
        aria-expanded={ this.props.ariaExpanded }
        // onMouseEnter={ this.onMouseEnter }
        // onMouseOver={ this.onMouseOver }
        // onMouseLeave={ this.onMouseLeave }
        onMouseDown={ this.onMouseDown }
        onMouseUp={ this.onMouseUp }
        className={ rootClass }
        tabIndex={ this.determineTabIndex() }
      >
        { this.props.children }
      </div>
    );

    // return (
    //   <FocusManager
    //     onFocusEnter={ this.onFocusEnter }
    //     onFocusWithin={ this.onFocusWithin }
    //     onFocusLeave={ this.onFocusLeave }
    //   >
    //     <div
    //       role={ this.props.role }
    //       ref={ this.setRef }
    //       style={ this.props.style }
    //       aria-label={ this.props.ariaLabel }
    //       aria-expanded={ this.props.ariaExpanded }
    //       onMouseEnter={ this.onMouseEnter }
    //       onMouseOver={ this.onMouseOver }
    //       onMouseLeave={ this.onMouseLeave }
    //       onMouseDown={ this.onMouseDown }
    //       onMouseUp={ this.onMouseUp }
    //       className={ rootClass }
    //       tabIndex={ this.determineTabIndex() }
    //     >
    //       { this.props.children }
    //     </div>
    //   </FocusManager>
    // );
  }
}
